import { Data } from '../../app/data'
import { Entity } from '../../app/entity'
import { Storage } from '../../app/storage'
import { Repo } from '../../app/repo'
import { RepoSQLite } from '../../app/repo/sqlite'
import { StorageSQLite } from '../../app/storage/sqlite'
import { assert } from 'chai'

class ItemData extends Data {
    uniqKey(): string {
        return 'uid'
    }

    uid: string
    title: string
}
class ItemEntity extends Entity<ItemData> {
    _getRepo(storage: Storage): Repo<Entity<ItemData>> {
        return undefined
    }

    _getVO(): ItemData {
        return new ItemData()
    }
}
class ItemsRepo extends RepoSQLite<ItemEntity> {
    Entity(): ItemEntity {
        return new ItemEntity()
    }

    Name(): string {
        return 'items'
    }

    async Create(): Promise<void> {
        const st = this._storage as StorageSQLite
        await st?.db?.run(`
			create table if not exists ${this.Name()} (
                uid varchar(255) not null unique,
				title text null
            )
        `)
    }
}

describe('ItemsRepoSQLite', () => {
    const dsn = __dirname + `/sqlite.test.db`
    let storage: StorageSQLite
    let repo: ItemsRepo

    beforeAll(async () => {
        storage = new StorageSQLite({ dsn })
        await storage.connect()
        storage.db.on('trace', (data: string) => {
            console.log(data)
        })
        repo = new ItemsRepo(storage)
        await repo.Recreate()
        for (const uid of ['foo', 'bar', 'pew', 'baz', 'fox']) {
            const d = new ItemData()
            d.uid = uid
            const e = new ItemEntity(d)
            await repo.save(e)
        }
    })

    describe('#findById', () => {
        it('should return persisted entity', async () => {
            const e = await repo.findById('foo')
            assert.isTrue(e.isPersisted())
            assert.isDefined(e._id)
        })
    })
})
